#!/usr/bin/env node
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var awsSdk = require('aws-sdk');
var readline = require('readline');

const kms = new awsSdk.KMS();
const isBase64 = /^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{4}|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)$/;
const dictionary = new Map();
async function decrypt(ciphertext) {
    const result = await kms
        .decrypt({ CiphertextBlob: Buffer.from(ciphertext, 'base64') })
        .promise();
    const plaintext = result.Plaintext ? result.Plaintext.toString() : ciphertext;
    return dictionary.set(ciphertext, plaintext) && plaintext;
}
var kmsDecrypt = (ciphertext) => ciphertext.length === 0 ||
    process.env.DISABLE_AWS_KMS_THINGY ||
    !isBase64.test(ciphertext)
    ? String(ciphertext)
    :
        dictionary.get(ciphertext) ||
            decrypt(ciphertext);

const kms$1 = new awsSdk.KMS();
const dictionary$1 = new Map();
async function encrypt({ plaintext, keyId, }) {
    const result = await kms$1
        .encrypt({ KeyId: keyId, Plaintext: plaintext })
        .promise();
    const ciphertext = result.CiphertextBlob
        ? result.CiphertextBlob.toString('base64')
        : plaintext;
    return dictionary$1.set(plaintext, ciphertext) && ciphertext;
}
var kmsEncrypt = ({ plaintext, keyId, }) => process.env.DISABLE_AWS_KMS_THINGY
    ? String(plaintext)
    :
        dictionary$1.get(keyId + plaintext) ||
            encrypt({ plaintext, keyId });

async function encrypt$1(parameters) {
    return 'plaintext' in parameters
        ? kmsEncrypt(parameters)
        : Promise.all(parameters.map(kmsEncrypt));
}
async function decrypt$1(ciphertext) {
    return typeof ciphertext === 'undefined'
        ? undefined
        : typeof ciphertext === 'string'
            ? kmsDecrypt(ciphertext)
            : Promise.all(ciphertext.map(kmsDecrypt));
}

const COLUMNS = process.stdout.columns || 32;
const askQuestion = (question, prompt = '→', sensitive = false) => {
    const line = readline.createInterface({
        input: process.stdin,
        output: process.stdout,
    });
    return new Promise(resolve => line.question(`\n${question}:\n\n${prompt} `, answer => !line.close() &&
        (sensitive
            ? !readline.moveCursor(process.stdin, 0, -1) &&
                !readline.clearLine(process.stdin, 0) &&
                !console.log(prompt + '  ' + '*'.repeat(answer.length))
            : true) &&
        resolve(answer.trim())));
};
const printUsageText = () => console.info(`
Usage: aws-kms-thingy [command]

Commands:
  - encrypt       Encrypt a string with an AWS KMS key
  - decrypt       Decrypt ciphertext encrypted with AWS KMS
`);
const prettyResult = (title, text) => !console.log(`\n${'〰️'.repeat((COLUMNS - (title.length + 2)) / 4)} ${title} ${'〰️'.repeat((COLUMNS - (title.length + 2)) / 4)}\n`, text, `\n${'〰️'.repeat(COLUMNS / 2)}\n`) && text;
async function encrypt$2() {
    const keyId = process.env.AWS_KMS_KEY_ARN ||
        (await askQuestion('Please enter AWS KMS Key ARN to use'));
    const plaintext = await askQuestion('Enter plain text to encrypt', '🔓', true);
    const result = (await encrypt$1({ keyId, plaintext }));
    return prettyResult('🔐 Encrypted', result);
}
async function decrypt$2() {
    const ciphertext = await askQuestion('Enter ciphertext to decrypt', '🔐');
    const result = (await decrypt$1(ciphertext));
    return prettyResult('🔓 Decrypted', result);
}
async function main([, , action] = process.argv) {
    const actions = {
        decrypt: decrypt$2,
        encrypt: encrypt$2,
    };
    return action && action in actions
        ? !console.log(`aws-kms-thingy ${action}`) && actions[action]()
        : printUsageText();
}
main().catch(console.error);

exports.main = main;
